"""Dataset sizes for testing."""

DATA_TRAIN_SIZES = {
    "EqualMinimalJapaneseVowels": 20,
    "MinimalCardanoSentiment": 20,
    "MinimalChinatown": 20,
    "MinimalGasPrices": 20,
    "MinimalJapaneseVowels": 20,
    "ShampooSales": 24,
    "UnequalMinimalCardanoSentiment": 20,
    "UnequalMinimalChinatown": 20,
    "UnequalMinimalGasPrices": 20,
}

DATA_TEST_SIZES = {
    "EqualMinimalJapaneseVowels": 20,
    "MinimalCardanoSentiment": 20,
    "MinimalChinatown": 20,
    "MinimalGasPrices": 20,
    "MinimalJapaneseVowels": 20,
    "ShampooSales": 12,
    "UnequalMinimalCardanoSentiment": 20,
    "UnequalMinimalChinatown": 20,
    "UnequalMinimalGasPrices": 20,
}
